// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.27.1
// 	protoc        v3.11.4
// source: github.com/containerd/ttrpc/example/example.proto

package example

import (
	empty "github.com/golang/protobuf/ptypes/empty"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Method1Request struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Foo string `protobuf:"bytes,1,opt,name=foo,proto3" json:"foo,omitempty"`
	Bar string `protobuf:"bytes,2,opt,name=bar,proto3" json:"bar,omitempty"`
}

func (x *Method1Request) Reset() {
	*x = Method1Request{}
	if protoimpl.UnsafeEnabled {
		mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[0]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *Method1Request) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Method1Request) ProtoMessage() {}

func (x *Method1Request) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[0]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Method1Request.ProtoReflect.Descriptor instead.
func (*Method1Request) Descriptor() ([]byte, []int) {
	return file_github_com_containerd_ttrpc_example_example_proto_rawDescGZIP(), []int{0}
}

func (x *Method1Request) GetFoo() string {
	if x != nil {
		return x.Foo
	}
	return ""
}

func (x *Method1Request) GetBar() string {
	if x != nil {
		return x.Bar
	}
	return ""
}

type Method1Response struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Foo string `protobuf:"bytes,1,opt,name=foo,proto3" json:"foo,omitempty"`
	Bar string `protobuf:"bytes,2,opt,name=bar,proto3" json:"bar,omitempty"`
}

func (x *Method1Response) Reset() {
	*x = Method1Response{}
	if protoimpl.UnsafeEnabled {
		mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[1]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *Method1Response) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Method1Response) ProtoMessage() {}

func (x *Method1Response) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[1]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Method1Response.ProtoReflect.Descriptor instead.
func (*Method1Response) Descriptor() ([]byte, []int) {
	return file_github_com_containerd_ttrpc_example_example_proto_rawDescGZIP(), []int{1}
}

func (x *Method1Response) GetFoo() string {
	if x != nil {
		return x.Foo
	}
	return ""
}

func (x *Method1Response) GetBar() string {
	if x != nil {
		return x.Bar
	}
	return ""
}

type Method2Request struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Action string `protobuf:"bytes,1,opt,name=action,proto3" json:"action,omitempty"`
}

func (x *Method2Request) Reset() {
	*x = Method2Request{}
	if protoimpl.UnsafeEnabled {
		mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[2]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *Method2Request) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Method2Request) ProtoMessage() {}

func (x *Method2Request) ProtoReflect() protoreflect.Message {
	mi := &file_github_com_containerd_ttrpc_example_example_proto_msgTypes[2]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Method2Request.ProtoReflect.Descriptor instead.
func (*Method2Request) Descriptor() ([]byte, []int) {
	return file_github_com_containerd_ttrpc_example_example_proto_rawDescGZIP(), []int{2}
}

func (x *Method2Request) GetAction() string {
	if x != nil {
		return x.Action
	}
	return ""
}

var File_github_com_containerd_ttrpc_example_example_proto protoreflect.FileDescriptor

var file_github_com_containerd_ttrpc_example_example_proto_rawDesc = []byte{
	0x0a, 0x31, 0x67, 0x69, 0x74, 0x68, 0x75, 0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x63, 0x6f, 0x6e,
	0x74, 0x61, 0x69, 0x6e, 0x65, 0x72, 0x64, 0x2f, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2f, 0x65, 0x78,
	0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2e, 0x70, 0x72,
	0x6f, 0x74, 0x6f, 0x12, 0x10, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70,
	0x6c, 0x65, 0x2e, 0x76, 0x31, 0x1a, 0x1b, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2f, 0x70, 0x72,
	0x6f, 0x74, 0x6f, 0x62, 0x75, 0x66, 0x2f, 0x65, 0x6d, 0x70, 0x74, 0x79, 0x2e, 0x70, 0x72, 0x6f,
	0x74, 0x6f, 0x22, 0x34, 0x0a, 0x0e, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x31, 0x52, 0x65, 0x71,
	0x75, 0x65, 0x73, 0x74, 0x12, 0x10, 0x0a, 0x03, 0x66, 0x6f, 0x6f, 0x18, 0x01, 0x20, 0x01, 0x28,
	0x09, 0x52, 0x03, 0x66, 0x6f, 0x6f, 0x12, 0x10, 0x0a, 0x03, 0x62, 0x61, 0x72, 0x18, 0x02, 0x20,
	0x01, 0x28, 0x09, 0x52, 0x03, 0x62, 0x61, 0x72, 0x22, 0x35, 0x0a, 0x0f, 0x4d, 0x65, 0x74, 0x68,
	0x6f, 0x64, 0x31, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x10, 0x0a, 0x03, 0x66,
	0x6f, 0x6f, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52, 0x03, 0x66, 0x6f, 0x6f, 0x12, 0x10, 0x0a,
	0x03, 0x62, 0x61, 0x72, 0x18, 0x02, 0x20, 0x01, 0x28, 0x09, 0x52, 0x03, 0x62, 0x61, 0x72, 0x22,
	0x28, 0x0a, 0x0e, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x32, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73,
	0x74, 0x12, 0x16, 0x0a, 0x06, 0x61, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28,
	0x09, 0x52, 0x06, 0x61, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x32, 0x9e, 0x01, 0x0a, 0x07, 0x45, 0x78,
	0x61, 0x6d, 0x70, 0x6c, 0x65, 0x12, 0x4e, 0x0a, 0x07, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x31,
	0x12, 0x20, 0x2e, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65,
	0x2e, 0x76, 0x31, 0x2e, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x31, 0x52, 0x65, 0x71, 0x75, 0x65,
	0x73, 0x74, 0x1a, 0x21, 0x2e, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70,
	0x6c, 0x65, 0x2e, 0x76, 0x31, 0x2e, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x31, 0x52, 0x65, 0x73,
	0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x43, 0x0a, 0x07, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x32,
	0x12, 0x20, 0x2e, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65,
	0x2e, 0x76, 0x31, 0x2e, 0x4d, 0x65, 0x74, 0x68, 0x6f, 0x64, 0x31, 0x52, 0x65, 0x71, 0x75, 0x65,
	0x73, 0x74, 0x1a, 0x16, 0x2e, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2e, 0x70, 0x72, 0x6f, 0x74,
	0x6f, 0x62, 0x75, 0x66, 0x2e, 0x45, 0x6d, 0x70, 0x74, 0x79, 0x42, 0x2d, 0x5a, 0x2b, 0x67, 0x69,
	0x74, 0x68, 0x75, 0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x63, 0x6f, 0x6e, 0x74, 0x61, 0x69, 0x6e,
	0x65, 0x72, 0x64, 0x2f, 0x74, 0x74, 0x72, 0x70, 0x63, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c,
	0x65, 0x3b, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x62, 0x06, 0x70, 0x72, 0x6f, 0x74, 0x6f,
	0x33,
}

var (
	file_github_com_containerd_ttrpc_example_example_proto_rawDescOnce sync.Once
	file_github_com_containerd_ttrpc_example_example_proto_rawDescData = file_github_com_containerd_ttrpc_example_example_proto_rawDesc
)

func file_github_com_containerd_ttrpc_example_example_proto_rawDescGZIP() []byte {
	file_github_com_containerd_ttrpc_example_example_proto_rawDescOnce.Do(func() {
		file_github_com_containerd_ttrpc_example_example_proto_rawDescData = protoimpl.X.CompressGZIP(file_github_com_containerd_ttrpc_example_example_proto_rawDescData)
	})
	return file_github_com_containerd_ttrpc_example_example_proto_rawDescData
}

var file_github_com_containerd_ttrpc_example_example_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_github_com_containerd_ttrpc_example_example_proto_goTypes = []interface{}{
	(*Method1Request)(nil),  // 0: ttrpc.example.v1.Method1Request
	(*Method1Response)(nil), // 1: ttrpc.example.v1.Method1Response
	(*Method2Request)(nil),  // 2: ttrpc.example.v1.Method2Request
	(*empty.Empty)(nil),     // 3: google.protobuf.Empty
}
var file_github_com_containerd_ttrpc_example_example_proto_depIdxs = []int32{
	0, // 0: ttrpc.example.v1.Example.Method1:input_type -> ttrpc.example.v1.Method1Request
	0, // 1: ttrpc.example.v1.Example.Method2:input_type -> ttrpc.example.v1.Method1Request
	1, // 2: ttrpc.example.v1.Example.Method1:output_type -> ttrpc.example.v1.Method1Response
	3, // 3: ttrpc.example.v1.Example.Method2:output_type -> google.protobuf.Empty
	2, // [2:4] is the sub-list for method output_type
	0, // [0:2] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_github_com_containerd_ttrpc_example_example_proto_init() }
func file_github_com_containerd_ttrpc_example_example_proto_init() {
	if File_github_com_containerd_ttrpc_example_example_proto != nil {
		return
	}
	if !protoimpl.UnsafeEnabled {
		file_github_com_containerd_ttrpc_example_example_proto_msgTypes[0].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*Method1Request); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_github_com_containerd_ttrpc_example_example_proto_msgTypes[1].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*Method1Response); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_github_com_containerd_ttrpc_example_example_proto_msgTypes[2].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*Method2Request); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_github_com_containerd_ttrpc_example_example_proto_rawDesc,
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_github_com_containerd_ttrpc_example_example_proto_goTypes,
		DependencyIndexes: file_github_com_containerd_ttrpc_example_example_proto_depIdxs,
		MessageInfos:      file_github_com_containerd_ttrpc_example_example_proto_msgTypes,
	}.Build()
	File_github_com_containerd_ttrpc_example_example_proto = out.File
	file_github_com_containerd_ttrpc_example_example_proto_rawDesc = nil
	file_github_com_containerd_ttrpc_example_example_proto_goTypes = nil
	file_github_com_containerd_ttrpc_example_example_proto_depIdxs = nil
}
